#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# khard documentation build configuration file, created by
# sphinx-quickstart on Sun Jan 14 10:35:27 2018.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import pathlib
import sys
sys.path.insert(0, os.path.abspath('../..'))
from khard.version import version as original_version


# update the template file for the docs if necessary
def update_template_file():
    here = pathlib.Path(__file__).parent
    src = here.parent.parent/'khard'/'data'/'template.yaml'
    dest = here/'examples'/'template.yaml'
    if not dest.exists() or src.stat().st_ctime > dest.stat().st_ctime:
        dest.write_text(src.read_text().format(
            "\n    Jabber  : \n    Skype   : \n    Twitter : "))


update_template_file()
del update_template_file


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'autoapi.extension',  # https://sphinx-autoapi.readthedocs.io/en/latest/
    'sphinx.ext.autodoc',  # https://pypi.org/project/sphinx-autodoc-typehints/
    'sphinx.ext.autosectionlabel',
    'sphinx.ext.todo',
    'sphinx_autodoc_typehints',  # https://pypi.org/project/sphinx-autodoc-typehints/
]

autoapi_type = 'python'
autoapi_dirs = ['../../khard']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'khard'
copyright = '2020, Eric Scheibler'
author = 'Eric Scheibler'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '.'.join(original_version.split('.')[0:2])
# The full version, including alpha/beta/rc tags.
release = original_version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'alabaster'

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# This is required for the alabaster theme
# refs: http://alabaster.readthedocs.io/en/latest/installation.html#sidebars
html_sidebars = {
    '**': [
        'relations.html',  # needs 'show_related': True theme option to display
        'searchbox.html',
    ]
}


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'kharddoc'


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('man/khard', 'khard', 'Console address book manager', '', 1),
    ('man/khard.conf', 'khard.conf', 'configuration file for khard', '', 5),
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'khard', 'khard Documentation',
     author, 'khard', 'One line description of project.',
     'Miscellaneous'),
]
